import ibmsecurity.utilities.tools
import logging

from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/services"
requires_modules = None
requires_version = None
requires_model = "Appliance"


def add(isamAppliance, service_name, address, active, port, weight, secure, ssllabel, check_mode=False, force=False):
    """
    Creating a server
    """
    check_exist, warnings = _check_exist(isamAppliance, service_name, address, port)

    if force is True or check_exist is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:

            return isamAppliance.invoke_post("Creating a server",
                                             "{0}/{1}/servers".format(module_uri, service_name),
                                             {
                                                 "active": active,
                                                 "address": address,
                                                 "port": port,
                                                 "weight": weight,
                                                 "secure": secure,
                                                 "ssllabel": ssllabel

                                             },
                                             requires_version=requires_version, requires_modules=requires_modules, requires_model=requires_model)
    else:
        return isamAppliance.create_return_object(warnings=warnings)


def delete(isamAppliance, service_name, address, port, check_mode=False, force=False):
    """
    deletes a server from specified service name
    """

    check_exist, warnings = _check_exist(isamAppliance, service_name, address, port)
    if force is True or check_exist is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            id = address + ":" + str(port)
            return isamAppliance.invoke_delete("Deleting a server",
                                               "{0}/{1}/servers/{2}".format(module_uri, service_name, id),
                                               requires_version=requires_version, requires_modules=requires_modules, requires_model=requires_model)

    else:
        return isamAppliance.create_return_object(warnings=warnings)


def get(isamAppliance, service_name, address, port, check_mode=False, force=False):
    """
    Retrieves server from specified service name
    """

    id = address + ":" + str(port)
    return (isamAppliance.invoke_get("Retrieving a server", "{0}/{1}/servers/{2}".format(module_uri, service_name, id),
                                 requires_version=requires_version, requires_modules=requires_modules, requires_model=requires_model))


def get_all(isamAppliance, service_name, check_mode=False, force=False):
    """
    Retrieves a list of servers under a specified service
    """
    return isamAppliance.invoke_get("Retrieving servers for a service",
                                    "{0}/{1}/servers".format(module_uri, service_name),
                                    requires_version=requires_version, requires_modules=requires_modules, requires_model=requires_model)


def update(isamAppliance, service_name, address, active, port, weight, secure=False, ssllabel=None, new_address=None, new_port=None, check_mode=False, force=False):
    """
    Updating server
    """
    id = address + ":" + str(port)
    json_data = {'active': active, 'secure': secure, 'ssllabel': ssllabel, 'weight': weight}
    if new_address is not None:
        json_data['address'] = new_address
    else:
        json_data['address'] = address

    if new_port is not None:
        json_data['port'] = new_port
    else:
        json_data['port'] = port
    change_required, warnings = _check_update(isamAppliance, service_name, address, port, json_data)

    if force is True or change_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:

            return isamAppliance.invoke_put("Updating a server",
                                            "{0}/{1}/servers/{2}".format(module_uri, service_name, id),
                                            json_data,
                                            requires_modules=requires_modules,
                                            requires_version=requires_version,
                                            requires_model = requires_model)
    else:
        return isamAppliance.create_return_object(warnings=warnings)


def _check_update(isamAppliance, service_name, address, port, json_data):
    """
    idempontency test
    """

    ret_obj = get(isamAppliance, service_name, address, port)
    warnings = ret_obj['warnings']

    ret_data = ret_obj['data']

    if 'id' in ret_data:
        del ret_data['id']
    else:
        return False, warnings

    sorted_ret_data = tools.json_sort(ret_data)
    sorted_json_data = tools.json_sort(json_data)
    logger.debug("Sorted Existing Data:{0}".format(sorted_ret_data))
    logger.debug("Sorted Desired  Data:{0}".format(sorted_json_data))

    if sorted_ret_data != sorted_json_data:
        return True, warnings
    else:
        return False, warnings


def _check_exist(isamAppliance, service_name, address, port):
    """
    idempotency test for delete function
    """

    id = address + ":" + str(port)
    ret_obj = get_all(isamAppliance, service_name)
    warnings = ret_obj['warnings']

    for obj in ret_obj['data']:
        if obj['id'] == id:
            return True, warnings

    return False, warnings


def compare(isamAppliance1, service_name1, isamAppliance2, service_name2):
    """
    Compare cluster configuration between two appliances
    """
    ret_obj1 = get_all(isamAppliance1, service_name1)
    ret_obj2 = get_all(isamAppliance2, service_name2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging
from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)

# URI for this module
uri = "/mail_object"


def get(isvgAppliance, check_mode=False, force=False):
    """
    Retrieve mail configuration
    """
    return isvgAppliance.invoke_get("Retrieve mail configuration entries", "{0}".format(uri))


def search(isvgAppliance, name="Mail Configuration", check_mode=False, force=False):
    """
    Search for existing mail configuration.
    """
    ret_obj = get(isvgAppliance)
    return_obj = isvgAppliance.create_return_object()

    for obj in ret_obj['data']:
        if 'name' in obj and obj['name'] == name:
            logger.info("Found mail entry: {0}".format(obj['name']))
            return_obj['data'] = obj
            return_obj['rc'] = 0

    return return_obj


def add(isvgAppliance, mailFrom, mailServer, mailUser, mailPwd, mailBaseUrl, port, useSSL, check_mode=False, force=False):
    """
    Updating mail configuration
    """
    if force is True:
        if check_mode is True:
            return isvgAppliance.create_return_object(changed=True)
        else:
            return isvgAppliance.invoke_post(
                "Configure mail configuration", "{0}".format(uri),
                    {
                      "mailFrom": mailFrom,
                      "mailServer": mailServer,
                      "mailUser": mailUser,
                      "mailPwd": mailPwd,
                      "mailBaseUrl": mailBaseUrl,
                      "name":"Mail Configuration",
                      "port": port,
                      "useSSL": useSSL,
                      "_isNew": True,
                      "action": "configure"
                    })

    return isvgAppliance.create_return_object(changed=False)


def delete(isvgAppliance, name="Mail Configuration", check_mode=False, force=False):
    """
    Un-configure mail configuration
    """
    ret_obj = search(isvgAppliance, name)
    warnings = ret_obj['warnings']

    if force is True or ret_obj['data'] != {}:
        if check_mode is True:
            return isvgAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            ret_obj['data'] = ret_obj['data']
            uuid = ret_obj['data']['uuid']
            return isvgAppliance.invoke_delete(
                "Un-configure mail configuration", "{0}/{1}".format(uri, uuid), warnings=warnings)

    return isvgAppliance.create_return_object(warnings=warnings)


def update(isvgAppliance, mailFrom, mailServer, mailUser, mailPwd, mailBaseUrl, port, useSSL, check_mode=False, force=False):
    """
    Updating mail configuration
    """
    ret_obj = get(isvgAppliance)
    warnings = ret_obj['warnings']

    # JSON payload of interest is at first (and only) position of array
    ret_obj['data'] = ret_obj['data'][0]

    uuid = ret_obj['data']['uuid']    

    needs_update = False

    # Create a simple json with just the attributes
    json_data = {
        "name": "Mail Configuration",
        "uuid": uuid
    }
   
    if 'lastModified' in ret_obj['data']:
        del ret_obj['data']['lastModified']
    if 'certCheckSum' in ret_obj['data']:
        del ret_obj['data']['certCheckSum']
    
    # mandatory attributes
    if mailFrom is not None:
        json_data['mailFrom'] = mailFrom
    elif 'mailFrom' in ret_obj['data']:
        if ret_obj['data']['mailFrom'] is not None:
            json_data['mailFrom'] = ret_obj['data']['mailFrom']
        else:
            del ret_obj['data']['mailFrom']
    if mailServer is not None:
        json_data['mailServer'] = mailServer
    elif 'mailServer' in ret_obj['data']:
        if ret_obj['data']['mailServer'] is not None:
            json_data['mailServer'] = ret_obj['data']['mailServer']
        else:
            del ret_obj['data']['mailServer']
    if port is not None:
        json_data['port'] = port
    elif 'port' in ret_obj['data']:
        if ret_obj['data']['port'] is not None:
            json_data['port'] = ret_obj['data']['port']
        else:
            del ret_obj['data']['port']
    if useSSL is not None:
        json_data['useSSL'] = useSSL
    elif 'useSSL' in ret_obj['data']:
        if ret_obj['data']['useSSL'] is not None:
            json_data['useSSL'] = ret_obj['data']['useSSL']
        else:
            del ret_obj['data']['useSSL']
    # optional attributes
    if mailUser is not None:
        json_data['mailUser'] = mailUser
    elif 'mailUser' in ret_obj['data']:
        if ret_obj['data']['mailUser'] is not None:
            json_data['mailUser'] = ret_obj['data']['mailUser']
        else:
            del ret_obj['data']['mailUser']
    if mailPwd is not None:
        json_data['mailPwd'] = mailPwd
    elif 'mailPwd' in ret_obj['data']:
        if ret_obj['data']['mailPwd'] is not None:
            json_data['mailPwd'] = ret_obj['data']['mailPwd']
        else:
            del ret_obj['data']['mailPwd']
    if mailBaseUrl is not None:
        json_data['mailBaseUrl'] = mailBaseUrl
    elif 'mailBaseUrl' in ret_obj['data']:
        if ret_obj['data']['mailBaseUrl'] is not None:
            json_data['mailBaseUrl'] = ret_obj['data']['mailBaseUrl']
        else:
            del ret_obj['data']['mailBaseUrl']

    sorted_ret_obj = tools.json_sort(ret_obj['data'])
    sorted_json_data = tools.json_sort(json_data)
    logger.debug("Sorted Existing Data:{0}".format(sorted_ret_obj))
    logger.debug("Sorted Desired  Data:{0}".format(sorted_json_data))
    if sorted_ret_obj != sorted_json_data:
        json_data['_isNew'] = False
        json_data['action'] = "reconfigure"
        needs_update = True

    if force is True or needs_update is True:
        if check_mode is True:
            return isvgAppliance.create_return_object(changed=True)
        else:
            return isvgAppliance.invoke_put(
                "Update mail configuration", "{0}/{1}".format(uri, uuid),
                json_data, warnings=warnings)

    return isvgAppliance.create_return_object(changed=False)


def set(isvgAppliance, mailFrom, mailServer, mailUser=None, mailPwd=None, mailBaseUrl=None, port=25, useSSL=False, check_mode=False, force=False):
    """
    Creating or Modifying a mail configuration
    """
    name = "Mail Configuration"
    if (search(isvgAppliance, name))['data'] == {}:
        # Force the add - we already know object does not exist
        logger.info("Mail entry {0} had no match, requesting to configure.".format(name))
        return add(isvgAppliance, mailFrom, mailServer, mailUser, mailPwd, mailBaseUrl, port, useSSL, check_mode=check_mode, force=True)
    else:
        # Update request
        logger.info("Mail entry {0} exists, requesting to reconfigure.".format(name))
        return update(isvgAppliance, mailFrom, mailServer, mailUser, mailPwd, mailBaseUrl, port, useSSL, check_mode=check_mode, force=force)



import logging

logger = logging.getLogger(__name__)

# URI for this module
uri = "/events/system"
requires_modules = None
requires_version = None


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Exporting the System Event Log

    """
    return isamAppliance.invoke_get("Exporting the System Event Log",
                                    "{0}".format(uri),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get(isamAppliance, field, filter, check_mode=False, force=False):
    """
    Exporting the filtered System Event Log

    """
    return isamAppliance.invoke_get("Exporting the filtered System Event Log",
                                    "{0}/{1}/{2}".format(uri, field, filter),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get_by_date_range_filter(isamAppliance, time_filter, check_mode=False, force=False):
    """
    Exporting the filtered System Event Log Using a Date Range

    """
    return isamAppliance.invoke_get("Exporting the filtered System Event Log Using a Date Range",
                                    "{0}/time/{1}".format(uri, time_filter,
                                                          requires_modules=requires_modules,
                                                          requires_version="10.0.1"))



import logging
from ibmsecurity.utilities import tools
import os.path

logger = logging.getLogger(__name__)

uri = "/support"


def get(isdsAppliance, check_mode=False, force=False):
    """
    Retrieving information about all valid support files
    """
    return isdsAppliance.invoke_get("Retrieving information about all valid support files", uri)


def create(isdsAppliance, comment='', check_mode=False, force=False):
    """
    Creating a new support file
    """
    if force is True or _check(isdsAppliance, comment=comment) is False:
        if check_mode is True:
            return isdsAppliance.create_return_object(changed=True)
        else:
            return isdsAppliance.invoke_post("Creating a new support file", uri,
                                             {
                                                 'comment': comment
                                             })

    return isdsAppliance.create_return_object()


def _check(isdsAppliance, comment='', id=None):
    """
    Check if the last created support file has the exact same comment or id exists

    :param isdsAppliance:
    :param comment:
    :return:
    """
    ret_obj = get(isdsAppliance)

    if id != None:
        for sups in ret_obj['data']:
            if sups['id'] == id:
                return True
    else:
        for sups in ret_obj['data']:
            if sups['comment'] == comment:
                return True

    #       if isinstance(ret_obj['data'], list):
    #           # check only latest comment
    #           sup_file = min(ret_obj['data'], key=lambda sup: sup['index'])
    #           if sup_file['comment'] == comment:
    #               return True

    return False


def delete(isdsAppliance, id, check_mode=False, force=False):
    """
    Deleting a support file
    """
    if force is True or _check(isdsAppliance, id=id) is True:
        if check_mode is True:
            return isdsAppliance.create_return_object(changed=True)
        else:
            if isinstance(id, list):
                del_uri = "{0}/multi_destroy{1}".format(uri, tools.create_query_string(record_ids=','.join(id)))
            else:
                del_uri = "{0}/{1}".format(uri, id)
            return isdsAppliance.invoke_delete("Deleting a support file", del_uri)

    return isdsAppliance.create_return_object()


def modify(isdsAppliance, id, filename, comment, check_mode=False, force=False):
    """
    Modify the snapshot comment
    """
    if force is True or _check(isdsAppliance, id=id) is True:
        if check_mode is True:
            return isdsAppliance.create_return_object(changed=True)
        else:
            return isdsAppliance.invoke_put("Modifying snapshot", "{0}/{1}".format(uri, id),
                                            {
                                                'id': id,
                                                'filename': filename,
                                                'comment': comment
                                            })

    return isdsAppliance.create_return_object()


def download(isdsAppliance, filename, id, check_mode=False, force=False):
    """
    Download snapshot file(s) to a zip file.
    Note: id can be a list or a single value
    """
    if force is True or (_check(isdsAppliance, id=id) is True and os.path.exists(filename) is False):
        if check_mode is False:  # No point downloading a file if in check_mode
            if isinstance(id, list):
                id = ','.join(id)
            uri_download = "{0}/download{1}".format(uri, tools.create_query_string(record_ids=id))
            return isdsAppliance.invoke_get_file("Downloading snapshots", uri_download, filename)

    return isdsAppliance.create_return_object()


def download_latest(isdsAppliance, dir='.', check_mode=False, force=False):
    """
    Download latest support file to a zip file.
    """
    ret_obj = get(isdsAppliance)

    # Get snapshot with lowest 'id' value - that will be latest one
    sup_file = min(ret_obj['data'], key=lambda sup: sup['index'])
    id = sup_file['id']
    file = sup_file['filename']
    filename = os.path.join(dir, file)

    return download(isdsAppliance, filename, id, check_mode, force)


def compare(isdsAppliance1, isdsAppliance2):
    """
    Compare list of support files between 2 appliances - not sure if this will ever be useful?
    """
    ret_obj1 = get(isdsAppliance1)
    ret_obj2 = get(isdsAppliance2)

    for snapshot in ret_obj1['data']:
        del snapshot['id']
        del snapshot['filename']

    for snapshot in ret_obj2['data']:
        del snapshot['id']
        del snapshot['filename']

    return tools.json_compare(ret_obj1, ret_obj2, deleted_keys=['id', 'filename'])



import logging
from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)

# URI for this module
uri = "/wga/kerberos/config/capaths"
requires_modules = ['wga']
requires_version = None


def get_all(isamAppliance, recursive='yes', includeValues='yes', addParent='yes', check_mode=False, force=False):
    """
    Retrieve Kerberos Configuration: CA Paths
    """
    return isamAppliance.invoke_get("Retrieve Kerberos Configuration: CA Paths",
                                    "{0}{1}".format(uri, tools.create_query_string(recursive=recursive,
                                                                                   includeValues=includeValues,
                                                                                   addParent=addParent)),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get(isamAppliance, name, check_mode=False, force=False):
    """
   Retrieve a specific Kerberos Client Realm Configuration
    """
    return isamAppliance.invoke_get("Retrieve Kerberos Configuration: CA Paths",
                                    "{0}/{1}".format(uri, name),
                                    requires_modules=requires_modules, requires_version=requires_version)


def search(isamAppliance, name, check_mode=False, force=False):
    """
    Search client realm by name
    """
    ret_obj = get_all(isamAppliance)
    return_obj = isamAppliance.create_return_object()
    return_obj["warnings"] = ret_obj["warnings"]

    for obj in ret_obj['data']:
        if obj['name'] == name:
            logger.info("Found name {0} id: {1}".format(name, obj['id']))
            return_obj['data'] = obj['id']

    return return_obj


def add(isamAppliance, name, check_mode=False, force=False):
    """
    Add a client realm entry
    """
    ret_obj = search(isamAppliance, name=name, check_mode=check_mode, force=force)
    warnings = ret_obj["warnings"]

    if force is True or ret_obj["data"] == {}:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_post(
                "Create an Kerberos Configuration Client realm entry", uri,
                {
                    "subsection": name
                }, requires_modules=requires_modules, requires_version=requires_version, warnings=warnings)

    return isamAppliance.create_return_object(warnings=warnings)


def delete(isamAppliance, name, check_mode=False, force=False):
    """
    Delete an client realm entry
    """
    ret_obj = search(isamAppliance, name, check_mode=check_mode, force=force)
    warnings = ret_obj["warnings"]

    if ret_obj['data'] != {} or force is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_delete(
                "Delete an Kerberos Configuration Client Realm entry",
                "{0}/{1}".format(uri, name), requires_modules=requires_modules,
                requires_version=requires_version, warnings=warnings)

    return isamAppliance.create_return_object(warnings=warnings)


def compare(isamAppliance1, isamAppliance2):
    """
    Compare kerberos configuration ca paths between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    for obj in ret_obj1['data']:
        del obj['id']
    for obj in ret_obj2['data']:
        del obj['id']

    return tools.json_compare(ret_obj1, ret_obj2, deleted_keys=['id'])



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)


def get(isamAppliance, directory_name, check_mode=False, force=False):
    """
    Retrieving the list of suffixes for a particular federated directory
    """
    return isamAppliance.invoke_get("Retrieving the list of suffixes for a particular federated directory",
                                    "/isam/runtime_components/federated_directories/{0}/suffix/v1".format(
                                        directory_name))


def add(isamAppliance, directory_name, suffix, use_ssl=False, client_cert_label=None,
        check_mode=False,
        force=False):
    """
    Create a new suffix in a particular federated directory
    """
    if force is True or _check(isamAppliance, directory_name, suffix) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post(
                "Create a new suffix in a particular federated directory",
                "/isam/runtime_components/federated_directories/{0}/suffix/v1".format(directory_name),
                {
                    'suffix': suffix
                })

    return isamAppliance.create_return_object()


def delete(isamAppliance, directory_name, suffix_name, check_mode=False, force=False):
    """
    Remove an existing suffix from a federated directory
    """
    if force is True or _check(isamAppliance, directory_name, suffix_name) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete(
                "Remove an existing suffix from a federated directory",
                "/isam/runtime_components/federated_directories/{0}/suffix/{1}/v1".format(directory_name, suffix_name))

    return isamAppliance.create_return_object()


def _check(isamAppliance, directory_name, suffix):
    """
    Check if federated directory suffix exists - will return true if any match is found

    :param isamAppliance:
    :param directory_name:
    :param suffix:
    :return:
    """
    ret_obj = get(isamAppliance, directory_name)

    for suffix_obj in ret_obj['data']:
        if isinstance(suffix, list):  # Add passes a list
            for new_suffix in suffix:
                if new_suffix['id'] == suffix_obj['id']:
                    return True
        else:  # Update passes just suffix_name
            if suffix_obj['id'] == suffix:
                return True

    return False


def compare(isamAppliance1, isamAppliance2, directory_name):
    """
    Compare snmp objects between two appliances
    """
    ret_obj1 = get(isamAppliance1, directory_name)
    ret_obj2 = get(isamAppliance2, directory_name)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging
import ibmsecurity.utilities.tools
import os.path

logger = logging.getLogger(__name__)

requires_modules = ["wga"]
requires_version = "10.0.5.0"


def get(isamAppliance, instance_name, check_mode=False, force=False):
    """
    Retrieving the WAF configuration file (modsecurity.conf)
    """
    return isamAppliance.invoke_get("Retrieving the WAF configuration file",
     "/isam/advanced_configuration/{0}?component=waf".format(instance_name))


def export_file(isamAppliance, instance_name, filename, check_mode=False, force=False):
    """
    Exporting the WAF configuration file (modsecurity.conf)
    """
    if force is True or os.path.exists(filename) is False:
        if check_mode is False:
            return isamAppliance.invoke_get_file(
            "Exporting the WAF configuration file",
            "/isam/advanced_configuration/{0}?export&component=waf".format(instance_name),
            filename)

    return isamAppliance.create_return_object()


def revert(isamAppliance, instance_name, check_mode=False, force=False):
    """
    Reverting a previously updated WAF configuration file
    """
    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:
        return isamAppliance.invoke_put(
            "Reverting a previously updated WAF configuration file",
            "/isam/advanced_configuration/{0}&component=waf".format(instance_name),
            {
                'operation': 'revert'
            })


def update(isamAppliance, instance_name, file_contents, check_mode=False, force=False):
    """
    Updating the WAF configuration file
    """
    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:
        return isamAppliance.invoke_put(
            "Updating the WAF configuration file",
            "/isam/advanced_configuration/{0}?component=waf".format(instance_name),
            {
                'file_contents': file_contents
            })


def compare(isamAppliance1, isamAppliance2, resource_id):
    """
    Compare stanzas within resource between two appliances
    """
    ret_obj1 = get(isamAppliance1, resource_id)
    ret_obj2 = get(isamAppliance2, resource_id)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1=ret_obj1, ret_obj2=ret_obj2, deleted_keys=[])



import logging

logger = logging.getLogger(__name__)

requires_model = "Appliance"


def export(isamAppliance, filename, type="db2", check_mode=False, force=False):
    """
    Export the configuration database
    """
    import os.path

    if force is True or os.path.exists(filename) is False:
        if check_mode is False:  # No point downloading a file if in check_mode
            return isamAppliance.invoke_get_file(
                "Export the configuration database",
                "/isam/cluster/configdb/v1?type={0}".format(type),
                filename,
                requires_model=requires_model,
            )

    return isamAppliance.create_return_object()



import logging
import ibmsecurity.utilities.tools
import os.path

logger = logging.getLogger(__name__)


def get(isamAppliance, resource_id, check_mode=False, force=False):
    """
    Retrieving a runtime configuration file
    """
    return isamAppliance.invoke_get("Retrieving a runtime configuration file",
                                    "/isam/advanced_configuration/{0}".format(resource_id))


def export_file(isamAppliance, resource_id, filename, check_mode=False, force=False):
    """
    Exporting a runtime configuration file
    """
    if force is True or os.path.exists(filename) is False:
        if check_mode is False:
            return isamAppliance.invoke_get_file(
                "Exporting a runtime configuration file",
                "/isam/advanced_configuration/{0}?export".format(resource_id), filename)

    return isamAppliance.create_return_object()


def revert(isamAppliance, resource_id, check_mode=False, force=False):
    """
    Reverting a previously updated runtime configuration file
    """
    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:
        return isamAppliance.invoke_put(
            "Reverting a previously updated runtime configuration file",
            "/isam/advanced_configuration/{0}".format(resource_id),
            {
                'operation': 'revert'
            })


def update(isamAppliance, resource_id, file_contents, check_mode=False, force=False):
    """
    Updating a runtime configuration file
    """
    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:
        return isamAppliance.invoke_put(
            "Updating a runtime configuration file",
            "/isam/advanced_configuration/{0}".format(resource_id),
            {
                'file_contents': file_contents
            })


def import_file(isamAppliance, resource_id, filename, check_mode=False, force=False):
    """
    Updating a runtime configuration file
    """
    import os
    if os.path.exists(filename) is False:
        warnings = []
        warnstr = filename + " does not exist."
        warnings.append(warnstr)
        return isamAppliance.create_return_object(warnings=warnings)

    with open(filename, 'r') as infile:
        file_contents = infile.read()

    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:
        return isamAppliance.invoke_put(
            "Updating a runtime configuration file",
            "/isam/advanced_configuration/{0}".format(resource_id),
            {
                'file_contents': file_contents
            })


def compare(isamAppliance1, isamAppliance2, resource_id):
    """
    Compare stanzas within resource between two appliances
    """
    ret_obj1 = get(isamAppliance1, resource_id)
    ret_obj2 = get(isamAppliance2, resource_id)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1=ret_obj1, ret_obj2=ret_obj2, deleted_keys=[])



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)


def get_all(isdsAppliance, check_mode=False, force=False):
    """
    Retrieving all interfaces
    :rtype: (str, dict)
    """
    return isdsAppliance.invoke_get("Retrieving all interfaces", "/widgets/mgmtinterface")


def get_all_app(isdsAppliance, check_mode=False, force=False):
    """
    Retrieving all application interfaces
    :rtype: (str, dict)
    """
    return isdsAppliance.invoke_get("Retrieving all application interfaces", "/application_interfaces")


def get(isdsAppliance, uuid, check_mode=False, force=False):
    """
    Retrieving a single interface
    """
    return isdsAppliance.invoke_get("Retrieving a single interface", "/application_interfaces/" + uuid + "/addresses/1")


def add(isdsAppliance, address, netmask, network_type, ipFqdn, prefix="", interface="P.1", check_mode=False,
        force=False):
    """
    Adding an address to an interface
    """
    if check_mode is True:
        return isdsAppliance.create_return_object(changed=True)
    else:
        return isdsAppliance.invoke_post(
            "Creating a new address for an interface",
            "/application_interfaces/" + interface + "/addresses",
            {
                'address': address,
                'netmask': netmask,
                'type': network_type,
                'ipFqdn': ipFqdn
            })


def compare(isdsAppliance1, isdsAppliance2):
    """
    Compare interfaces between 2 appliances
    """
    ret_obj1 = get_all(isdsAppliance1)
    ret_obj2 = get_all(isdsAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2)



import logging
import ibmsecurity.utilities.tools
import os.path

logger = logging.getLogger(__name__)


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Get information on existing users
    """
    return isamAppliance.invoke_get("Retrieving users", "/sysaccount/users/v1")


def get(isamAppliance, id, check_mode=False, force=False):
    """
    Get information on particular user by id
    """
    return isamAppliance.invoke_get("Retrieving user", "/sysaccount/users/{0}/v1".format(id))


def create(isamAppliance, id, password, groups=[], check_mode=False, force=False):
    """
    Create a new user
    """
    if force is True or _check(isamAppliance, id=id) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post("Creating user", "/sysaccount/users/v1",
                                             {
                                                 'id': id,
                                                 'password': password,
                                                 'groups': groups
                                             })

    return isamAppliance.create_return_object()


def _check(isamAppliance, id=None):
    """
    Check if the last created user has the exact same id or id exists

    :param isamAppliance:
    :param comment:
    :return:
    """
    ret_obj = get_all(isamAppliance)

    if id != None:
        for users in ret_obj['data']:
            if users['id'] == id:
                return True

    return False


def delete(isamAppliance, id, check_mode=False, force=False):
    """
    Delete a user
    """
    if force is True or _check(isamAppliance, id=id) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Deleting user", "/sysaccount/users/{0}/v1".format(id))

    return isamAppliance.create_return_object()


def modify(isamAppliance, id, password, old_password=None, check_mode=False, force=False):
    """
    Change a users password
    """
    warnings = []
    if old_password is not None and old_password != '':
        warnings.append("old_password is not required. Provided value will be ignored.")
    if force is True or _check(isamAppliance, id=id) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_put("Change password", "/sysaccount/users/{0}/v1".format(id),
                                            {
                                                'password': password
                                            }, warnings=warnings)

    return isamAppliance.create_return_object()


def modify_self(isamAppliance, password, old_password, check_mode=False, force=False):
    """
    Change the current user's password
    """
    if force is True or password != old_password:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Change password", "/sysaccount/self/v1",
                                            {
                                                'password': password,
                                                'old_password': old_password
                                            })

    return isamAppliance.create_return_object()


def compare(isamAppliance1, isamAppliance2):
    """
    Compare the list of users between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2)



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)

requires_model = "Appliance"

def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the SNMP Monitoring configuration
    """
    return isamAppliance.invoke_get("Retrieving the SNMP Monitoring configuration", "/snmp/v1", requires_model=requires_model)


def set_v1v2(isamAppliance, community, port=161, check_mode=False, force=False):
    """
    Set SNMP Monitoring v1/2
    """
    snmpv1v2c = {
        "community": community
    }
    return set(isamAppliance, True, port, snmpv1v2c, None, check_mode, force)


def set_v3(isamAppliance, securityLevel, securityUser, authProtocol, authPassword, privacyProtocol, privacyPassword,
           port=161, check_mode=False, force=False):
    """
    Set SNMP Monitoring v3
    """
    snmpv3 = {
        "securityLevel": securityLevel,
        "securityUser": securityUser,
        "authProtocol": authProtocol,
        "authPassword": authPassword,
        "privacyProtocol": privacyProtocol,
        "privacyPassword": privacyPassword
    }
    return set(isamAppliance, True, port, None, snmpv3, check_mode, force)


def disable(isamAppliance, check_mode=False, force=False):
    """
    Disable SNMP Monitoring
    """
    return set(isamAppliance, False, None, None, None, check_mode, force)


def set(isamAppliance, enabled, port=None, snmpv1v2c=None, snmpv3=None, check_mode=False, force=False):
    """
    Updating the SNMP Monitoring configuration
    """
    warnings = []
    if snmpv1v2c is not None and snmpv1v2c != '' and snmpv3 is not None and snmpv3 != '':
        warnings.append("SNMP v1v2 and v3 settings cannot be specified at the same time. Ignoring v1v2 settings.")
        snmpv1v2c = None

    update_required, json_data, warn_str = _check_all(isamAppliance, snmpv1v2c, snmpv3, port, enabled)
    warnings = warnings + warn_str

    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_put(
                "Updating the SNMP Monitoring configuration",
                "/snmp/v1", json_data, warnings=warnings)

    return isamAppliance.create_return_object(warnings=warnings)


def _check_all(isamAppliance, snmpv1v2c, snmpv3, port, enabled):
    json_data = {
        "enabled": enabled
    }
    if port is not None and port != '':
        json_data['port'] = port
    if snmpv1v2c is not None and snmpv1v2c != '':
        json_data['snmpv1v2c'] = snmpv1v2c
    if snmpv3 is not None and snmpv3 != '':
        json_data['snmpv3'] = snmpv3
    update_required = False

    ret_obj = get(isamAppliance)
    warnings = ret_obj['warnings']
    if warnings != []:
        if 'Docker' in warnings[0]:
            return update_required, json_data, warnings

    import ibmsecurity.utilities.tools
    sorted_json_data = ibmsecurity.utilities.tools.json_sort(json_data)
    logger.debug("Sorted input: {0}".format(sorted_json_data))
    sorted_ret_obj = ibmsecurity.utilities.tools.json_sort(ret_obj['data'])
    logger.debug("Sorted existing data: {0}".format(sorted_ret_obj))
    if sorted_ret_obj != sorted_json_data:
        logger.info("Changes detected, update needed.")
        update_required = True

    return update_required, json_data, warnings


def _check(isamAppliance, community, port):
    ret_obj = get(isamAppliance)

    if ret_obj['data']['enabled'] is False:
        logger.info("SNMP Monitoring is not enabled")
        return False

    if ret_obj['data']['snmpv1v2c']['community'] != community:
        logger.info("SNMP Monitoring community is different")
        return False

    if ret_obj['data']['port'] != port:
        logger.info("SNMP Monitoring port is different")
        return False

    return True


def compare(isamAppliance1, isamAppliance2):
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging
import ibmsecurity.isam.aac.server_connections.ws
from ibmsecurity.utilities.tools import json_sort

logger = logging.getLogger(__name__)

uri = "/mga/scim/configuration"
requires_modules = ["mga", "federation"]
requires_version = "9.0.2"


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the current group SCIM configuration settings
    """
    return isamAppliance.invoke_get("Retrieving the current group SCIM configuration settings",
                                    "{0}/urn:ietf:params:scim:schemas:core:2.0:Group".format(uri),
                                    requires_modules=requires_modules,
                                    requires_version=requires_version
                                    )

def set(isamAppliance, ldap_object_classes, group_dn, check_mode=False, force=False):
    """
    Updating the group SCIM configuration settings
    """
    obj = get(isamAppliance)
    obj = obj['data']['urn:ietf:params:scim:schemas:core:2.0:Group']

    current_dn = obj['group_dn']
    current_classes = obj['ldap_object_classes']

    if current_dn == group_dn:
        sorted_new = json_sort(ldap_object_classes)
        sorted_old = json_sort(current_classes)
        if sorted_new == sorted_old:
            update_required = False
        else:
            update_required = True
    else:
        update_required = True

    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            data = {}
            data['group_dn'] = group_dn
            data['ldap_object_classes'] = ldap_object_classes
            return isamAppliance.invoke_put("Updating the group SCIM configuration settings",
                                            "{0}/urn:ietf:params:scim:schemas:core:2.0:Group".format(uri),
                                            data,
                                            requires_modules=requires_modules,
                                            requires_version=requires_version
                                            )

    return isamAppliance.create_return_object(changed=False)


import logging
import ibmsecurity.utilities.tools
import json

logger = logging.getLogger(__name__)

def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the administrator settings
    """
    return isamAppliance.invoke_get("Retrieving the administrator settings", "/admin_cfg")


def set_pw(isamAppliance, oldPassword, newPassword, sessionTimeout="30", httpsPort=None, check_mode=False, force=False):
    """
    Set password for admin user (super user for appliance)

    Note: this function is being maintained for backward compatibility. Use set() going forward.
    """
    warnings = ["Password change requested - cannot query existing password for idempotency check."]
    if check_mode is True:
        return isamAppliance.create_return_object(changed=True, warnings=warnings)
    else:
        json_data = {
            "oldPassword": oldPassword,
            "newPassword": newPassword,
            "confirmPassword": newPassword,
            "sessionTimeout": sessionTimeout
        }
        if httpsPort is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.1.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, httpsPort not supported. Needs 9.0.1.0 or higher. Ignoring httpsPort for this call.")
            else:
                json_data['httpsPort'] = httpsPort
        else:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.1.0") < 0:
                pass  # Can safely ignore httpsPort
            else:
                warnings.append("Default httpsPort of 443 will be set on the appliance.")
        return isamAppliance.invoke_put("Setting admin password", "/core/admin_cfg", json_data, warnings=warnings)


def set(isamAppliance, oldPassword=None, newPassword=None, minHeapSize=None, maxHeapSize=None, sessionTimeout=30,
        httpPort=None, httpsPort=None, minThreads=None, maxThreads=None, maxPoolSize=None, lmiDebuggingEnabled=None,
        consoleLogLevel=None, acceptClientCerts=None, validateClientCertIdentity=None, excludeCsrfChecking=None,
        enableSSLv3=None, maxFiles=None, maxFileSize=None, enabledTLS=None, sshdPort=None, sessionCachePurge=None,
        sessionInactivityTimeout=None, sshdClientAliveInterval=None, swapFileSize=None, httpProxy=None,
        enabledServerProtocols=None, loginHeader=None, loginMessage=None, pendingChangesLifetime=None,
        baSessionTimeout=None, httpsProxy=None, accessLogFormat=None, lmiMessageTimeout=None, validVerifyDomains=None,
        check_mode=False, force=False):
    """
    Updating the administrator settings
    """
    warnings = []

    update_required, warnings, json_data = _check(isamAppliance, oldPassword, newPassword, minHeapSize, maxHeapSize,
                                                  sessionTimeout, httpPort, httpsPort, minThreads, maxThreads,
                                                  maxPoolSize, lmiDebuggingEnabled, consoleLogLevel,
                                                  acceptClientCerts, validateClientCertIdentity,
                                                  excludeCsrfChecking, enableSSLv3, maxFiles, maxFileSize,
                                                  enabledTLS, sshdPort, sessionCachePurge, sessionInactivityTimeout,
                                                  sshdClientAliveInterval, swapFileSize, httpProxy,
                                                  enabledServerProtocols, loginHeader, loginMessage,
                                                  pendingChangesLifetime, baSessionTimeout, httpsProxy, accessLogFormat,
                                                  lmiMessageTimeout, validVerifyDomains, warnings)

    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_put(
                "Updating the administrator settings",
                "/core/admin_cfg", json_data, warnings=warnings)

    return isamAppliance.create_return_object(changed=False, warnings=warnings)


def _check(isamAppliance, oldPassword, newPassword, minHeapSize, maxHeapSize,
           sessionTimeout, httpPort, httpsPort, minThreads, maxThreads,
           maxPoolSize, lmiDebuggingEnabled, consoleLogLevel,
           acceptClientCerts, validateClientCertIdentity,
           excludeCsrfChecking, enableSSLv3, maxFiles, maxFileSize,
           enabledTLS, sshdPort, sessionCachePurge, sessionInactivityTimeout,
           sshdClientAliveInterval, swapFileSize, httpProxy,
           enabledServerProtocols, loginHeader, loginMessage, pendingChangesLifetime,
           baSessionTimeout, httpsProxy, accessLogFormat,
           lmiMessageTimeout, validVerifyDomains,
           warnings):
    """
    Check whether target key has already been set with the value
    :param isamAppliance:
    :param key:
    :param value:
    :return: True/False
    """
    ret_obj = get(isamAppliance)

    json_data = {
        "sessionTimeout": sessionTimeout
    }
    if oldPassword is not None:
        json_data["oldPassword"] = oldPassword
        if newPassword is None:
            warnings.append("Please provide new password, when old password is specified.")
    if newPassword is not None:
        warnings.append("Password change requested - cannot query existing password for idempotency check.")
        json_data["newPassword"] = newPassword
        json_data["confirmPassword"] = newPassword
        if oldPassword is None:
            warnings.append("Please provide old password, when new password is specified.")
    if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.1.0") < 0:
        if minHeapSize is not None or maxHeapSize is not None or httpPort is not None or httpsPort is not None or \
                minThreads is not None or maxThreads is not None or maxPoolSize is not None or \
                lmiDebuggingEnabled is not None or consoleLogLevel is not None or \
                acceptClientCerts is not None or validateClientCertIdentity is not None or \
                excludeCsrfChecking is not None or enableSSLv3 is not None or maxFiles is not None or \
                maxFileSize is not None or enabledTLS is not None or sshdPort is not None or \
                enabledServerProtocols is not None:
            warnings.append(
                "Appliance at version: {0}, only oldPassword, newPassword, sessionTimeout are supported. Needs 9.0.1.0 or higher. Ignoring other attributes for this call.")
    else:
        if minHeapSize is not None:
            json_data["minHeapSize"] = int(minHeapSize)
        elif 'minHeapSize' in ret_obj['data']:
            del ret_obj['data']['minHeapSize']
        if maxHeapSize is not None:
            json_data["maxHeapSize"] = int(maxHeapSize)
        elif 'maxHeapSize' in ret_obj['data']:
            del ret_obj['data']['maxHeapSize']
        if httpPort is not None:
            json_data["httpPort"] = int(httpPort)
        elif 'httpPort' in ret_obj['data']:
            del ret_obj['data']['httpPort']
        if httpsPort is not None:
            json_data["httpsPort"] = int(httpsPort)
        elif 'httpsPort' in ret_obj['data']:
            del ret_obj['data']['httpsPort']
        if minThreads is not None:
            json_data["minThreads"] = int(minThreads)
        elif 'minThreads' in ret_obj['data']:
            del ret_obj['data']['minThreads']
        if maxThreads is not None:
            json_data["maxThreads"] = int(maxThreads)
        elif 'maxThreads' in ret_obj['data']:
            del ret_obj['data']['maxThreads']
        if maxPoolSize is not None:
            json_data["maxPoolSize"] = int(maxPoolSize)
        elif 'maxPoolSize' in ret_obj['data']:
            del ret_obj['data']['maxPoolSize']
        if lmiDebuggingEnabled is not None:
            json_data["lmiDebuggingEnabled"] = lmiDebuggingEnabled
        elif 'lmiDebuggingEnabled' in ret_obj['data']:
            del ret_obj['data']['lmiDebuggingEnabled']
        if consoleLogLevel is not None:
            json_data["consoleLogLevel"] = consoleLogLevel
            if 'consoleLogLevel' in ret_obj['data'] and ret_obj['data']['consoleLogLevel'] == 'OFF':
                ret_obj['data']['consoleLogLevel'] = 'OFF'
        elif 'consoleLogLevel' in ret_obj['data']:
            del ret_obj['data']['consoleLogLevel']
        if acceptClientCerts is not None:
            json_data["acceptClientCerts"] = acceptClientCerts
        elif 'acceptClientCerts' in ret_obj['data']:
            del ret_obj['data']['acceptClientCerts']
        if validateClientCertIdentity is not None:
            json_data["validateClientCertIdentity"] = validateClientCertIdentity
        elif 'validateClientCertIdentity' in ret_obj['data']:
            del ret_obj['data']['validateClientCertIdentity']
        if excludeCsrfChecking is not None:
            json_data["excludeCsrfChecking"] = excludeCsrfChecking
        elif 'excludeCsrfChecking' in ret_obj['data']:
            del ret_obj['data']['excludeCsrfChecking']
        if enableSSLv3 is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "10.0.3.0") >= 0:
                warnings.append(
                    "Appliance at version: {0}, enableSSLv3: {1} is not supported. Needs max. 10.0.2.0. Ignoring for this call.".format(
                        isamAppliance.facts["version"], enableSSLv3))
            else:
                json_data["enableSSLv3"] = enableSSLv3
        elif 'enableSSLv3' in ret_obj['data']:
            del ret_obj['data']['enableSSLv3']
        if maxFiles is not None:
            json_data["maxFiles"] = int(maxFiles)
        elif 'maxFiles' in ret_obj['data']:
            del ret_obj['data']['maxFiles']
        if maxFileSize is not None:
            json_data["maxFileSize"] = int(maxFileSize)
        elif 'maxFileSize' in ret_obj['data']:
            del ret_obj['data']['maxFileSize']
        if sshdPort is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.3.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, sshdPort: {1} is not supported. Needs 9.0.3.0 or higher. Ignoring sshdPort for this call.".format(
                        isamAppliance.facts["version"], sshdPort))
            else:
                json_data["sshdPort"] = int(sshdPort)
        elif 'sshdPort' in ret_obj['data']:
            del ret_obj['data']['sshdPort']
        if enabledTLS is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.4.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, enabledTLS: {1} is not supported. Needs 9.0.4.0 or higher. Ignoring enabledTLS for this call.".format(
                        isamAppliance.facts["version"], enabledTLS))
            else:
                json_data["enabledTLS"] = enabledTLS
        elif 'enabledTLS' in ret_obj['data']:
            del ret_obj['data']['enabledTLS']
        if sessionCachePurge is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.5.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, sessionCachePurge: {1} is not supported. Needs 9.0.5.0 or higher. Ignoring sessionCachePurge for this call.".format(
                        isamAppliance.facts["version"], sessionCachePurge))
            else:
                json_data["sessionCachePurge"] = int(sessionCachePurge)
        elif 'sessionCachePurge' in ret_obj['data']:
            del ret_obj['data']['sessionCachePurge']
        if sessionInactivityTimeout is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.5.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, sessionInactivityTimeout: {1} is not supported. Needs 9.0.5.0 or higher. Ignoring sessionInactivityTimeout for this call.".format(
                        isamAppliance.facts["version"], sessionInactivityTimeout))
            else:
                json_data["sessionInactivityTimeout"] = int(sessionInactivityTimeout)
        elif 'sessionInactivityTimeout' in ret_obj['data']:
            del ret_obj['data']['sessionInactivityTimeout']
        if sshdClientAliveInterval is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.5.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, sshdClientAliveInterval: {1} is not supported. Needs 9.0.5.0 or higher. Ignoring sshdClientAliveInterval for this call.".format(
                        isamAppliance.facts["version"], sshdClientAliveInterval))
            else:
                json_data["sshdClientAliveInterval"] = int(sshdClientAliveInterval)
        elif 'sshdClientAliveInterval' in ret_obj['data']:
            del ret_obj['data']['sshdClientAliveInterval']
        if swapFileSize is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.5.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, swapFileSize: {1} is not supported. Needs 9.0.5.0 or higher. Ignoring swapFileSize for this call.".format(
                        isamAppliance.facts["version"], swapFileSize))
            else:
                json_data["swapFileSize"] = swapFileSize
        elif 'swapFileSize' in ret_obj['data']:
            del ret_obj['data']['swapFileSize']
        if httpProxy is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.5.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, httpProxy: {1} is not supported. Needs 9.0.5.0 or higher. Ignoring httpProxy for this call.".format(
                        isamAppliance.facts["version"], httpProxy))
            else:
                json_data["httpProxy"] = httpProxy
        elif 'httpProxy' in ret_obj['data']:
            del ret_obj['data']['httpProxy']
        if enabledServerProtocols is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.7.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, enabledServerProtocols: {1} is not supported. Needs 9.0.7.0 or higher. Ignoring enabledServerProtocols for this call.".format(
                        isamAppliance.facts["version"], enabledServerProtocols))
            else:
                json_data["enabledServerProtocols"] = enabledServerProtocols
        elif 'enabledServerProtocols' in ret_obj['data']:
            del ret_obj['data']['enabledServerProtocols']
        if loginHeader is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.7.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, loginHeader: {1} is not supported. Needs 9.0.7.0 or higher. Ignoring loginHeader for this call.".format(
                        isamAppliance.facts["version"], loginHeader))
            else:
                json_data["loginHeader"] = loginHeader
        elif 'loginHeader' in ret_obj['data']:
            del ret_obj['data']['loginHeader']
        if loginMessage is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.7.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, loginMessage: {1} is not supported. Needs 9.0.7.0 or higher. Ignoring loginMessage for this call.".format(
                        isamAppliance.facts["version"], loginMessage))
            else:
                json_data["loginMessage"] = loginMessage
        elif 'loginMessage' in ret_obj['data']:
            del ret_obj['data']['loginMessage']
        if pendingChangesLifetime is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.7.0") < 0:
                warnings.append(
                    "Appliance at version: {0}, pendingChangesLifetime: {1} is not supported. Needs 9.0.7.0 or higher. Ignoring pendingChangesLifetime for this call.".format(
                        isamAppliance.facts["version"], pendingChangesLifetime))
            else:
                json_data["pendingChangesLifetime"] = pendingChangesLifetime
        elif 'pendingChangesLifetime' in ret_obj['data']:
            del ret_obj['data']['pendingChangesLifetime']
        if httpsProxy is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.7.0") < 0:
                warnings.append("Appliance at version: {0}, httpsProxy: {1} is not supported. Needs 9.0.7.0 or higher. Ignoring for this call.".format(
                    isamAppliance.facts["version"], httpsProxy))
            else:
                json_data["httpsProxy"] = httpsProxy
        elif 'httpsProxy' in ret_obj['data']:
            del ret_obj['data']['httpsProxy']
        #10.0.2 or something.  Also, when Python 3?
        # baSessionTimeout, httpsProxy, accessLogFormat, lmiMessageTimeout, validVerifyDomains,
        if baSessionTimeout is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "10.0.2.0") < 0:
                warnings.append("Appliance at version: {0}, baSessionTimeout: {1} is not supported. Needs 10.0.2.0 or higher. Ignoring for this call.".format(
                    isamAppliance.facts["version"], baSessionTimeout))
            else:
                json_data["baSessionTimeout"] = int(baSessionTimeout)
        elif 'baSessionTimeout' in ret_obj['data']:
            del ret_obj['data']['baSessionTimeout']
        if accessLogFormat is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "10.0.0.0") < 0:
                warnings.append("Appliance at version: {0}, accessLogFormat: {1} is not supported. Needs 10.0.0.0 or higher. Ignoring for this call.".format(
                    isamAppliance.facts["version"], accessLogFormat))
            else:
                json_data["accessLogFormat"] = accessLogFormat
        elif 'accessLogFormat' in ret_obj['data']:
            del ret_obj['data']['accessLogFormat']
        if lmiMessageTimeout is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "10.0.2.0") < 0:
                warnings.append("Appliance at version: {0}, lmiMessageTimeout: {1} is not supported. Needs 10.0.2.0 or higher. Ignoring for this call.".format(
                    isamAppliance.facts["version"], lmiMessageTimeout))
            else:
                json_data["lmiMessageTimeout"] = int(lmiMessageTimeout)
        elif 'lmiMessageTimeout' in ret_obj['data']:
            del ret_obj['data']['lmiMessageTimeout']
        if validVerifyDomains is not None:
            if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "10.0.2.0") < 0:
                warnings.append("Appliance at version: {0}, validVerifyDomains: {1} is not supported. Needs 10.0.2.0 or higher. Ignoring for this call.".format(
                    isamAppliance.facts["version"], validVerifyDomains))
            else:
                json_data["validVerifyDomains"] = validVerifyDomains
        elif 'validVerifyDomains' in ret_obj['data']:
            del ret_obj['data']['validVerifyDomains']
    sorted_ret_obj = json.dumps(ret_obj['data'], skipkeys=True, sort_keys=True)
    sorted_json_data = json.dumps(json_data, skipkeys=True, sort_keys=True)
    logger.debug("Sorted Existing Data:{0}".format(sorted_ret_obj))
    logger.debug("Sorted Desired  Data:{0}".format(sorted_json_data))
    if sorted_ret_obj != sorted_json_data:
        logger.debug("Admin Settings are found to be different. See above JSON for difference.")
        # Ensure users know how REST API handles httpsPort default value
        if httpsPort is None and ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"],
                                                                             "9.0.1.0") >= 0:
            warnings.append("Default httpsPort of 443 will be set on the appliance.")
        return True, warnings, json_data
    else:  # No changes required
        return False, warnings, json_data


def compare(isamAppliance1, isamAppliance2):
    """
    Compare advanced tuning parameters between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2)



import logging
from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)

# URI for this module
uri = "/wga/reverseproxy_logging/common"
requires_model = "Appliance"


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the names of all common log files and file sizes
    """
    return isamAppliance.invoke_get("Retrieving the names of all common log files and file sizes",
                                    uri, requires_model=requires_model)


def get(isamAppliance, file_id, options=None, size=None, start=None, check_mode=False, force=False):
    """
    Retrieving a snippet of a common log file
    """
    return isamAppliance.invoke_get("Retrieving a snippet of a common log file",
                                    "{0}/{1}{2}".format(uri, file_id, tools.create_query_string(
                                        options=options, start=start, size=size)), requires_model=requires_model)


def delete(isamAppliance, file_id, check_mode=False, force=False):
    """
    Clearing a common log file
    """
    ret_obj = {'warnings': ''}

    try:
        ret_obj = get(isamAppliance, file_id, start=1, size=1)
        delete_required = True  # Exception thrown if the file is empty

        # Check for Docker
        warnings = ret_obj['warnings']
        if warnings and 'Docker' in warnings[0]:
            return isamAppliance.create_return_object(warnings=ret_obj['warnings'])
    except:
        delete_required = False

    if force is True or delete_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=ret_obj['warnings'])
        else:
            return isamAppliance.invoke_delete(
                "Clearing a common log file",
                "{0}/{1}".format(uri, file_id), requires_model=requires_model)

    return isamAppliance.create_return_object(warnings=ret_obj['warnings'])


def export_file(isamAppliance, file_id, filename, check_mode=False, force=False):
    """
    Exporting a common log file
    """
    import os.path
    ret_obj = {'warnings': ''}

    if force is True or (os.path.exists(filename) is False):
        if check_mode is False:  # No point downloading a file if in check_mode
            return isamAppliance.invoke_get_file(
                "Exporting a common log file",
                "{0}/{1}?export".format(uri, file_id),
                filename, requires_model=requires_model)

    return isamAppliance.create_return_object(warnings=ret_obj['warnings'])



import logging
from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)

# URI for this module
uri = "/iam/access/v8/pips"
requires_modules = ["mga"]
requires_version = None


def get_all(isamAppliance, filter=None, sortBy=None, check_mode=False, force=False):
    """
    Retrieve a list of Policy Information Points (PIPs)

    """
    return isamAppliance.invoke_get("Retrieve a list of policy information points (pips)",
                                    "{0}/{1}".format(uri, tools.create_query_string(filter=filter, sortBy=sortBy)),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get(isamAppliance, name=None, check_mode=False, force=False):
    """
    Retrieve a specific Policy Information Point by name

    """
    ret_obj = search(isamAppliance, name, check_mode=False, force=False)
    id = ret_obj['data']

    if id == {}:
        logger.info("PIP '{0}' had no match, skipping retrieval.".format(name))
        warnings = ["PIP '{0}' had no match, skipping retrieval.".format(name)]
        return isamAppliance.create_return_object(warnings=warnings)
    else:
        return _get(isamAppliance, id)


def delete(isamAppliance, name=None, check_mode=False, force=False):
    """
    Delete a policy information point

    """
    ret_obj = search(isamAppliance, name, check_mode=False, force=False)
    id = ret_obj['data']

    if force is True or id != {}:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:

            return isamAppliance.invoke_delete(
                "Delete a policy information point",
                "{0}/{1}".format(uri, id),
                requires_modules=requires_modules, requires_version=requires_version
            )

    if id == {}:
        logger.info("PIP '{0}' does not exists, skipping delete.".format(name))
        return isamAppliance.create_return_object()

    return isamAppliance.create_return_object()


def compare(isamAppliance1, isamAppliance2):
    """
    Compare access policies between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    for obj in ret_obj1['data']:
        del obj['id']
    for obj in ret_obj2['data']:
        del obj['id']

    return tools.json_compare(ret_obj1, ret_obj2, deleted_keys=['id'])


def search(isamAppliance, name, force=False, check_mode=False):
    """
    Retrieve ID for named PIP
    """
    ret_obj = get_all(isamAppliance)

    ret_obj_new = isamAppliance.create_return_object()

    for obj in ret_obj['data']:
        if obj['name'] == name:
            logger.info("Found Policy '{0}' id: '{1}'".format(name, obj['id']))
            ret_obj_new['data'] = obj['id']

    return ret_obj_new


def _get(isamAppliance, id):
    return isamAppliance.invoke_get("Retrieve a specific PIP",
                                    "{0}/{1}".format(uri, id))


def _create_json(name, properties, attributes, description, type):
    json_data = {
        "name": name,
        "type": type,
    }

    if attributes is not None:
        json_data['attributes'] = attributes
    else:
        json_data['attributes'] = []

    if description is not None:
        json_data['description'] = description
    else:
        json_data['description'] = ''

    if properties is not None:
        json_data['properties'] = properties
    else:
        json_data['properties'] = []

    return json_data



import logging
from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)


def get_all(isamAppliance, start=None, count=None, filter=None, sortBy=None, check_mode=False, force=False):
    """
    Retrieve a list of authentication mechanisms types
    """
    return isamAppliance.invoke_get("Retrieve a list of authentication mechanisms types",
                                    "/iam/access/v8/authentication/mechanism/types{0}".format(
                                        tools.create_query_string(start=start, count=count, filter=filter,
                                                                  sortBy=sortBy)))


def get(isamAppliance, name, check_mode=False, force=False):
    """
    Retrieve a specific authentication mechanism type
    """
    ret_obj = search(isamAppliance, name=name, check_mode=check_mode, force=force)
    id = ret_obj['data']

    if id == {}:
        logger.info("Authentication Mechanism Type {0} had no match, skipping retrieval.".format(name))
        return isamAppliance.create_return_object()
    else:
        return isamAppliance.invoke_get("Retrieve a specific authentication mechanism type",
                                        "/iam/access/v8/authentication/mechanism/types/{0}".format(id))


def search(isamAppliance, name, force=False, check_mode=False):
    """
    Search authentication mechanism type ID by name
    """
    ret_obj = get_all(isamAppliance)
    return_obj = isamAppliance.create_return_object()

    for obj in ret_obj['data']:
        if obj['name'] == name:
            logger.info("Found Authentication Mechanism Type {0} id: {1}".format(name, obj['id']))
            return_obj['data'] = obj['id']
            return_obj['rc'] = 0

    return return_obj


def compare(isamAppliance1, isamAppliance2):
    """
    Compare Authentication Types between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    for obj in ret_obj1['data']:
        del obj['id']
    for obj in ret_obj2['data']:
        del obj['id']

    return tools.json_compare(ret_obj1, ret_obj2, deleted_keys=['id'])



import logging

logger = logging.getLogger(__name__)

uri = "/iam/access/v8/otp/config/rsa"
requires_modules = ["mga"]
requires_version = "8.0.0.0"


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieve a list of configuration files for RSA
    """
    return isamAppliance.invoke_get("Retrieve a list of configuration files for RSA",
                                    "{0}".format(uri),
                                    requires_modules=requires_modules, requires_version=requires_version)


def import_sdconf(isamAppliance, filepath, check_mode=False, force=False):
    """
    Import sdconf.rec

    """

    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:

        return isamAppliance.invoke_post_files(
            "Import sdconf.rec",
            "{0}/sdconf.rec".format(uri),
            [
                {
                    'file_formfield': 'file',
                    'filename': filepath,
                    'mimetype': 'application/file'
                }
            ],
            {},
            requires_modules=requires_modules, requires_version=requires_version
        )

    return isamAppliance.create_return_object()


def import_sdopts(isamAppliance, filepath, check_mode=False, force=False):
    """
    Import sdopts.rec

    """

    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:

        return isamAppliance.invoke_post_files(
            "Import sdopts.rec",
            "{0}/sdopts.rec".format(uri),
            [
                {
                    'file_formfield': 'file',
                    'filename': filepath,
                    'mimetype': 'application/file'
                }
            ],
            {},
            requires_modules=requires_modules, requires_version=requires_version
        )

    return isamAppliance.create_return_object()


def import_securid(isamAppliance, node, filepath, check_mode=False, force=False):
    """
    Import securid file for a node

    """

    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:

        return isamAppliance.invoke_post_files(
            "Import securid file for a node",
            "{0}/securid/{1}".format(uri, node),
            [
                {
                    'file_formfield': 'file',
                    'filename': filepath,
                    'mimetype': 'application/file'
                }
            ],
            {},
            requires_modules=requires_modules, requires_version=requires_version
        )

    return isamAppliance.create_return_object()


def delete_securid(isamAppliance, node, check_mode=False, force=False):
    """
    Delete the securid file from a node

    """

    ret_obj = get(isamAppliance)
    delete_required = False

    for obj in ret_obj['data']:
        if obj['fileName'] == "securid":
            if 'importTimestamp' in obj:
                delete_required = True

    if force is True or delete_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:

            return isamAppliance.invoke_delete(
                "Delete the securid file from a node",
                "{0}/securid/{1}".format(uri, node),
                requires_modules=requires_modules, requires_version=requires_version
            )

    return isamAppliance.create_return_object()


def delete_sdopts(isamAppliance, check_mode=False, force=False):
    """
    Delete sdopts.rec

    """

    ret_obj = get(isamAppliance)
    delete_required = False

    for obj in ret_obj['data']:
        if obj['fileName'] == "sdopts.rec":
            if 'importTimestamp' in obj:
                delete_required = True

    if force is True or delete_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:

            return isamAppliance.invoke_delete(
                "Delete sdopts.rec",
                "{0}/sdopts.rec".format(uri),
                requires_modules=requires_modules, requires_version=requires_version
            )

    return isamAppliance.create_return_object()


def delete_sdconf(isamAppliance, check_mode=False, force=False):
    """
    Delete sdconf.rec

    """

    ret_obj = get(isamAppliance)
    delete_required = False

    for obj in ret_obj['data']:
        if obj['fileName'] == "sdconf.rec":
            if 'importTimestamp' in obj:
                delete_required = True

    if force is True or delete_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:

            return isamAppliance.invoke_delete(
                "Delete sdconf.rec",
                "{0}/sdconf.rec".format(uri),
                requires_modules=requires_modules, requires_version=requires_version
            )

    return isamAppliance.create_return_object()



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)
requires_model = "Appliance"

def get_application_interfaces(isamAppliance, check_mode=False, force=False):
    """
    Retrieve a list of application interfaces
    """
    return isamAppliance.invoke_get("Retrieve a list of application interfaces",
                                    "/isam/wga_templates/interfaces.json",requires_model=requires_model)


def get_ip_addresses(isamAppliance, check_mode=False, force=False):
    """
    Listing IP addresses for all interfaces
    """
    return isamAppliance.invoke_get("Listing IP addresses for all interfaces",
                                    "/isam/wga_templates/ipaddress",requires_model=requires_model)


def get_next_http_port(isamAppliance, ip_address, check_mode=False, force=False):
    """
    Find the next available HTTP port for an interface
    """
    if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "8.0.0.0") < 0:
        get_uri = "/isam/wga_templates/httpport/{0}".format(ip_address)
    else:
        get_uri = "/isam/wga_templates/httpport?ip_addr={0}".format(ip_address)

    return isamAppliance.invoke_get("Find the next available HTTP port for an interface", get_uri,requires_model=requires_model)


def get_next_https_port(isamAppliance, ip_address, check_mode=False, force=False):
    """
    Find the next available HTTPS port for an interface
    """
    if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "8.0.0.0") < 0:
        get_uri = "/isam/wga_templates/httpsport/{0}".format(ip_address)
    else:
        get_uri = "/isam/wga_templates/httpsport?ip_addr={0}".format(ip_address)

    return isamAppliance.invoke_get("Find the next available HTTPS port for an interface", get_uri,requires_model=requires_model)


def get_defaults(isamAppliance, check_mode=False, force=False):
    """
    Get defaults
    """
    return isamAppliance.invoke_get("Get defaults", "/isam/wga_templates/defaults",requires_model=requires_model)



import logging

logger = logging.getLogger(__name__)
requires_model="Appliance"

try:
    basestring
except NameError:
    basestring = (str, bytes)


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieve the tracing levels
    """
    return isamAppliance.invoke_get("Retrieve the tracing levels",
                                    "/isam/cluster/tracing/v1", requires_model=requires_model)


def _check(isamAppliance, dsc):

    check_value,warnings = True,""
    ret_obj = get(isamAppliance)
    warnings = ret_obj['warnings']

    if isinstance(dsc, basestring):
        import ast
        dsc = ast.literal_eval(dsc)

    if 'dsc' in ret_obj['data']:
        check_value = (ret_obj['data']['dsc']==dsc)
        return check_value,warnings
    else:
        check_value=True
        return check_value,warnings


def set(isamAppliance, dsc, check_mode=False, force=False):
    """
    Updating the tracing levels
    """

    check_value,warnings = _check(isamAppliance, dsc)
    if force is True or check_value is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_put(
                "Updating the tracing levels",
                "/isam/cluster/tracing/v1",
                {
                    'dsc': dsc
                }, requires_model=requires_model)

    return isamAppliance.create_return_object(warnings=warnings)



import ibmsecurity.utilities.tools
import logging

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/services"
requires_modules = None
requires_version = None
requires_model = "Appliance"


def add(isamAppliance, enabled, name, address, port, netmask, interface, scheduler, health_check_interval,
        rise, fall, layer, attributes=[], servers=None, check_mode=False, force=False):
    """
    Creating a service
    """

    check_value, warnings = _check_add(isamAppliance, enabled, name, address, port, netmask, interface, scheduler,
                                   health_check_interval, rise, fall)
    if force is True or check_value is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_post("Creating a service", module_uri,
                                             {
                                                 "enabled": enabled,
                                                 "name": name,
                                                 "address": address,
                                                 "port": port,
                                                 "netmask": netmask,
                                                 "interface": interface,
                                                 "scheduler": scheduler,
                                                 "health_check_interval": health_check_interval,
                                                 "rise": rise,
                                                 "fall": fall,
                                                 "layer": layer,
                                                 "attributes": attributes,
                                                 "servers": servers

                                             }, requires_version=requires_version, requires_modules=requires_modules, requires_model=requires_model)
    else:
        return isamAppliance.create_return_object(warnings=warnings)


def delete(isamAppliance, service_name, check_mode=False, force=False):
    """
    deletes service
    """

    check_value, warnings = _check_delete(isamAppliance, service_name)
    if force is True or check_value is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_delete("Deleting a service", "{0}/{1}".format(module_uri, service_name),
                                               requires_version=requires_version, requires_modules=requires_modules, requires_model=requires_model)
    else:
        return isamAppliance.create_return_object(warnings=warnings)


def get(isamAppliance, service_name, check_mode=False, force=False):
    """
    Receives a single service
    """
    return isamAppliance.invoke_get(
        "Retrieving a service", "{0}/{1}".format(module_uri, service_name), requires_version=requires_version,
        requires_modules=requires_modules, requires_model=requires_model)


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Receives all services
    """
    return isamAppliance.invoke_get("Retrieving all service names", module_uri, requires_version=requires_version,
                                    requires_modules=requires_modules, requires_model=requires_model)


def update(isamAppliance, service_name, enabled, name, address, port, netmask, interface, scheduler,
           health_check_interval, rise, fall, check_mode=False, force=False):
    """
    updates specified service name
    """

    update_required, warnings = _check_update(isamAppliance, service_name, enabled, name, address, port, netmask, interface,
                                                scheduler, health_check_interval, rise, fall)
    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_put("Updating a service", "{0}/{1}".format(module_uri, service_name),
                                            {
                                                "address": address,
                                                "enabled": enabled,
                                                "fall": fall,
                                                "health_check_interval": health_check_interval,
                                                "interface": interface,
                                                "name": name,
                                                "netmask": netmask,
                                                "port": port,
                                                "rise": rise,
                                                "scheduler": scheduler

                                            }, requires_modules=requires_modules,
                                            requires_version=requires_version,
                                            requires_model = requires_model)
    else:
        return isamAppliance.create_return_object(warnings=warnings)


def _check_update(isamAppliance, service_name, enabled, name, address, port, netmask, interface, scheduler,
                  health_check_interval, rise, fall):

    warnings = ""
    try:
        ret_obj = get(isamAppliance, service_name)
        warnings =ret_obj['warnings']
    except:
        logger.warning("The requested service does not exist in the configuration.")
        check_value = False
        return check_value, warnings

    if 'enabled' in ret_obj['data']:
        if ret_obj['data']['enabled'] != enabled:
            return True, warnings
        if ret_obj['data']['address'] != address:
            return True, warnings
        if ret_obj['data']['fall'] != fall:
            return True, warnings
        if ret_obj['data']['health_check_interval'] != health_check_interval:
            return True, warnings
        if ret_obj['data']['interface'] != interface:
            return True, warnings
        if ret_obj['data']['name'] != name:
            return True, warnings
        if ret_obj['data']['netmask'] != netmask:
            return True, warnings
        if ret_obj['data']['port'] != port:
            return True, warnings
        if ret_obj['data']['rise'] != rise:
            return True, warnings
        if ret_obj['data']['scheduler'] != scheduler:
            return True, warnings


    return False, warnings


def _check_add(isamAppliance, enabled, name, address, port, netmask, interface, scheduler, health_check_interval,
               rise, fall):
    ret_obj = {}
    # checks to see if service exists
    warnings = ""
    try:
        ret_obj = get(isamAppliance, name)
        warnings = ret_obj['warnings']
    except:
        return True, warnings

    if 'name' in ret_obj['data']:
        if ret_obj['data']['name'] == name:
            logger.warning("Server Already Exist")
            return False, warnings
    else:
        return False, warnings

    if ret_obj['data'] != {}:
        if ret_obj['data']['enabled'] != enabled:
            return True, warnings
        if ret_obj['data']['name'] != name:
            return True, warnings
        if ret_obj['data']['address'] != address:
            return True, warnings
        if ret_obj['data']['port'] != port:
            return True, warnings
        if ret_obj['data']['netmask'] != netmask:
            return True, warnings
        if ret_obj['data']['interface'] != interface:
            return True, warnings
        if ret_obj['data']['scheduler'] != scheduler:
            return True, warnings
        if ret_obj['data']['health_check_interval'] != health_check_interval:
            return True, warnings
        if ret_obj['data']['fall'] != fall:
            return True, warnings
        if ret_obj['data']['rise'] != rise:
            return True, warnings


    return False, warnings


def _check_delete(isamAppliance, service_name):
    """
    idempotency test for delete function
    """
    ret_obj = {}
    warnings = ""

    try:
        ret_obj = get(isamAppliance, service_name)
        warnings = ret_obj['warnings']
    except:
        return False, warnings

    if 'enabled' in ret_obj['data']:
        if ret_obj['data']['enabled'] == True:
            return True, warnings

    return False, warnings


def compare(isamAppliance1, isamAppliance2):
    """
    Compare cluster configuration between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import random
import string
import logging
import pprint
import difflib
import hashlib
import ntpath
import re
from io import open
import zipfile
import json

logger = logging.getLogger(__name__)

try:
    basestring
except NameError:
    basestring = (str, bytes)


def json_sort(json_data):
    if isinstance(json_data, dict):
        return sorted((key, json_sort(value)) for key, value in json_data.items())

    if isinstance(json_data, list):
        return sorted(json_sort(x) for x in json_data)
    else:
        return json_data


class jsonSortedListEncoder(json.JSONEncoder):
   """
   This is a custom class for use with json.dumps
   https://docs.python.org/3/library/json.html
   """
   def encode(self, obj):
       def sort_lists(item):
           if isinstance(item, list):
               return sorted(sort_lists(i) for i in item)
           elif isinstance(item, dict):
               return {k: sort_lists(v) for k, v in item.items()}
           else:
               return item
       return super(jsonSortedListEncoder, self).encode(sort_lists(obj))


def json_replace_value(json_data, from_value, to_value):
    '''
    Replaces all found from_values with to_value in a json - from_value can be a JSON itself

    :param json_data:
    :param from_value:
    :param to_value:
    :return:
    '''
    if json_data == from_value:
        return to_value
    elif isinstance(json_data, dict):
        new_dict = {}
        for key, value in json_data.items():
            if type(value) == type(from_value) and value == from_value:
                value = to_value
            elif isinstance(value, dict):
                value = json_replace_value(value, from_value, to_value)
            elif isinstance(value, list):
                value = json_replace_value(value, from_value, to_value)
            new_dict[key] = value
        return new_dict
    elif isinstance(json_data, list):
        new_list = []
        for elem in json_data:
            if type(elem) == type(from_value) and elem == from_value:
                elem = to_value
            elif isinstance(elem, dict):
                elem = json_replace_value(elem, from_value, to_value)
            elif isinstance(elem, list):
                elem = json_replace_value(elem, from_value, to_value)
            new_list.append(elem)
        return new_list
    else:
        return json_data


def json_remove_value(json_data, value):
    '''
    Removes all found values (along with key if part of a dict)

    :param json_data:
    :param from_value:
    :param to_value:
    :return:
    '''
    if isinstance(json_data, dict):
        new_dict = {}
        for k, v in json_data.items():
            if isinstance(v, dict) or isinstance(v, list):
                v = json_remove_value(v, value)
                new_dict[k] = v
            elif type(v) == type(value) and v == value:
                pass
            else:
                new_dict[k] = v
        return new_dict
    elif isinstance(json_data, list):
        new_list = []
        for elem in json_data:
            if isinstance(elem, dict) or isinstance(elem, list):
                elem = json_remove_value(elem, value)
                new_list.append(elem)
            elif type(elem) == type(value) and elem == value:
                pass
            else:
                new_list.append(elem)
        return new_list
    else:
        if type(json_data) == type(value) and json_data == value:
            return None
        else:
            return json_data


def random_password(length=12, allow_special=True):
    myrg = random.SystemRandom()

    # If you want non-English characters, remove the [0:52]
    alphabet = string.ascii_letters[0:52] + string.digits
    if allow_special is True:
        alphabet = alphabet + "!@#$%^&*()"

    pw = str().join(myrg.choice(alphabet) for _ in range(length))

    return pw


def json_compare(ret_obj1, ret_obj2, deleted_keys=[]):
    ret_obj = {'rc': 0, 'data': {'matches': False, 'difference': '', 'deleted_keys': deleted_keys},
               'changed': False, 'warnings': []}

    # Pick the highest RC to return (should not matter either way)
    if ret_obj1['rc'] > ret_obj2['rc']:
        ret_obj['rc'] = ret_obj1['rc']
    else:
        ret_obj['rc'] = ret_obj2['rc']

    if 'warnings' in ret_obj1 and ret_obj1['warnings']:
        ret_obj['warnings'].append(ret_obj1['warnings'])
    if 'warnings' in ret_obj2 and ret_obj2['warnings']:
        ret_obj['warnings'].append(ret_obj2['warnings'])

    sorted_json1 = json_sort(ret_obj1['data'])
    sorted_json2 = json_sort(ret_obj2['data'])

    if sorted_json1 == sorted_json2:
        ret_obj['data']['matches'] = True
    else:
        psj1 = pprint.pformat(sorted_json1)
        logger.debug('Sorted JSON1 to Compare: \n' + psj1)
        psj2 = pprint.pformat(sorted_json2)
        logger.debug('Sorted JSON2 to Compare: \n' + psj2)
        diff = difflib.ndiff(psj1.split('\n'), psj2.split('\n'))
        ret_obj['data']['difference'] = '\n'.join(diff)
        ret_obj['data']['context_difference'] = list(difflib.context_diff(psj1.split('\n'), psj2.split('\n')))
        ret_obj['data']['html_difference'] = difflib.HtmlDiff().make_file(psj1.split('\n'), psj2.split('\n'), context=True)

    return ret_obj


def create_query_string(**kwargs):
    # Create a query string - using variable key value pairs
    query_str = ''
    for key in kwargs:
        if kwargs[key] is not None:
            if query_str == '':
                query_str = '?'
            else:
                query_str += '&'
            query_str += "{0}={1}".format(key, kwargs[key])

    logger.debug("Query Parameter to be used: {0}".format(query_str))

    return query_str


def files_same(original_file, new_file):
    """
    Compare two files
        -works with text, image, and zip files
    Returns Boolean
    """
    with open(original_file, 'rb') as f:
        original_file_contents = f.read()
    with open(new_file, 'rb') as f:
        new_file_contents = f.read()
    hash_original_file = hashlib.sha224(original_file_contents).hexdigest()
    hash_new_file = hashlib.sha224(new_file_contents).hexdigest()
    if hash_original_file == hash_new_file:
        return True
    else:
        return False

def files_same_zip_content(original_file, new_file):
    identical = True
    logger.debug("Comparing original_file[{}] vs new_file[{}]".format(original_file, new_file))
    z1 = zipfile.ZipFile(original_file)
    z2 = zipfile.ZipFile(new_file)
    
    if len(z1.infolist()) != len(z2.infolist()):
        logger.debug("number of archive elements differ: {} in {} vs {} from server".format(len(z1.infolist()), z1.filename, len(z2.infolist())))
        identical = False
        # Can stop comparison of zip files for perfomance
        return identical
    for zipentry in z1.infolist():
        if zipentry.filename not in z2.namelist():
            logger.debug("no file named {} found in {}".format(zipentry.filename, z2.filename))
            identical = False
        else:
            with z1.open(zipentry.filename) as f:
                original_file_contents = f.read()
            with z2.open(zipentry.filename) as f:
                new_file_contents = f.read()
            hash_original_file = hashlib.sha224(original_file_contents).hexdigest()
            hash_new_file = hashlib.sha224(new_file_contents).hexdigest()
            if hash_original_file != hash_new_file:
                identical = False
                logger.debug("content for zip file {} differs.".format(zipentry.filename))

    if identical:
        logger.info("content for zip files {} and {} are the same.".format(original_file,new_file))
    else:
        logger.info("content for zip files {} and {} are different.".format(original_file,new_file))

    return identical

def get_random_temp_dir():
    """
    Create a temporary directory
    """
    import os
    import tempfile
    tmpdir = tempfile.gettempdir()
    random_str = random_password(10, allow_special=False)
    tmpdir += '/%s' % random_str
    os.mkdir(tmpdir)
    return tmpdir


def strings(filename, min=4):
    """
    Emulate UNIX "strings" command on a file
    """
    with open(filename, 'r', errors='ignore') as f:
        result = ""
        for c in f.read():
            if c in string.printable:
                result += c
                continue
            if len(result) >= min:
                yield result
            result = ""
        if len(result) >= min:  # catch result at EOF
            yield result


def path_leaf(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)


def version_compare(version1, version2):
    """
    Compare two ISAM version strings. Please note that the versions should be all numeric separated by dots.

    Returns following values:
         0 - if version strings are equivalent
        >0 - if version1 is greater than version2
        <0 - if version1 is less than version2

    Test cases to run for verifying this code:
        assert version_compare("1", "1") == 0
        assert version_compare("2.1", "2.2") < 0
        assert version_compare("3.0.4.10", "3.0.4.2") > 0
        assert version_compare("4.08", "4.08.01") < 0
        assert version_compare("3.2.1.9.8144", "3.2") > 0
        assert version_compare("3.2", "3.2.1.9.8144") < 0
        assert version_compare("1.2", "2.1") < 0
        assert version_compare("2.1", "1.2") > 0
        assert version_compare("5.6.7", "5.6.7") == 0
        assert version_compare("1.01.1", "1.1.1") == 0
        assert version_compare("1.1.1", "1.01.1") == 0
        assert version_compare("1", "1.0") == 0
        assert version_compare("1.0", "1") == 0
        assert version_compare("1.0", "1.0.1") < 0
        assert version_compare("1.0.1", "1.0") > 0
        assert version_compare("1.0.2.0", "1.0.2") == 0
        assert version_compare("10.0", "9.0.3") > 0

    :param version1:
    :param version2:
    :return:
    """

    def normalize(v):
        v = re.sub(r'_b\d+$', '', v)
        return [int(x) for x in re.sub(r'(\.0+)*$', '', v).split(".")]

    if normalize(version1) == normalize(version2):
        return 0
    elif normalize(version1) > normalize(version2):
        return 1
    elif normalize(version1) < normalize(version2):
        return -1



import logging
import os.path
from ibmsecurity.utilities import tools
from ibmsecurity.utilities.tools import json_sort

logger = logging.getLogger(__name__)
uri = "/isam/authzserver"
requires_modules = None
requires_version = None
requires_model = "Appliance"


def get_all(isamAppliance, id, check_mode=False, force=False):
    """
    Retrieve the trace components and settings of an existing instance

    """
    return isamAppliance.invoke_get("Retrieve the trace components and settings of an existing instance",
                                    "{0}/{1}/tracing/v1".format(uri, id),
                                    requires_modules=requires_modules, requires_version=requires_version,
                                    requires_model=requires_model)


def get_list(isamAppliance, id, trace_id, check_mode=False, force=False):
    """
    List trace files for a trace component

    """
    return isamAppliance.invoke_get("List trace files for a trace component",
                                    "{0}/{1}/tracing/{2}/trace_files/v1".format(uri, id, trace_id),
                                    requires_modules=requires_modules, requires_version=requires_version,
                                    requires_model=requires_model)


def get(isamAppliance, id, trace_id, trace_file_id, size=None, start=None, options=None, check_mode=False, force=False):
    """
    Retrieve the trace file snippet for a trace component

    """
    return isamAppliance.invoke_get("Retrieve the trace file snippet for a trace component",
                                    "{0}/{1}/tracing/{2}/trace_files/{3}/v1{4}".format(uri, id, trace_id, trace_file_id,
                                                                                       tools.create_query_string(
                                                                                           size=size, start=start,
                                                                                           options=options)),
                                    requires_modules=requires_modules, requires_version=requires_version,
                                    requires_model=requires_model)


def export_file(isamAppliance, id, trace_id, trace_file_id, filepath, check_mode=False, force=False):
    """
    Export the trace file for a trace component
    """

    if os.path.exists(filepath) is True:
        logger.info("File '{0}' already exists.  Skipping export.".format(filepath))
        warnings = ["File '{0}' already exists.  Skipping export.".format(filepath)]
        return isamAppliance.create_return_object(warnings=warnings)

    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:
        return isamAppliance.invoke_get_file(
            "Export the trace file for a trace component",
            "{0}/{1}/tracing/{2}/trace_files/{3}/v1?export".format(uri, id, trace_id, trace_file_id),
            filepath, requires_model=requires_model)


def update(isamAppliance, id, trace_id, level, flush_interval, rollover_size, max_rollover_files, compress,
           check_mode=False, force=False):
    """
    Update the settings for a trace component
    """
    id_exists, warnings = _check_id(isamAppliance, id, trace_id, level, flush_interval,
                                    rollover_size, max_rollover_files, compress)

    if not warnings:
        if force is True or id_exists is False:
            if check_mode is True:
                return isamAppliance.create_return_object(changed=True, warnings=warnings)
            else:
                return isamAppliance.invoke_put(
                    "Update the settings for a trace component",
                    "{0}/{1}/tracing/{2}/v1".format(uri, id, trace_id),
                    {
                        'level': level,
                        'flush_interval': flush_interval,
                        'rollover_size': rollover_size,
                        'max_rollover_files': max_rollover_files,
                        'compress': compress
                    },
                    requires_model=requires_model
                )

    return isamAppliance.create_return_object(warnings=warnings)


def delete(isamAppliance, id, trace_id, trace_file_id, check_mode=False, force=False):
    """
    Delete Trace File for a trace component
    """

    id_exists, warnings = _check(isamAppliance, id, trace_id, trace_file_id)

    if not warnings:
        if force is True or id_exists is True:
            if check_mode is True:
                return isamAppliance.create_return_object(changed=True, warnings=warnings)
            else:
                return isamAppliance.invoke_delete(
                    "Delete Trace File for a trace component",
                    "{0}/{1}/tracing/{2}/trace_files/{3}/v1".format(uri, id, trace_id, trace_file_id),
                    requires_model=requires_model)

    return isamAppliance.create_return_object(warnings=warnings)


def _check_id(isamAppliance, id, trace_id, level, flush_interval, rollover_size, max_rollover_files, compress):
    """
    Check to see if the file_id exists or not
    """
    ret_obj = get_all(isamAppliance, id)
    id_exists, warnings = False, ret_obj['warnings']

    new_obj = {
        'id': trace_id,
        'level': level,
        'flush_interval': flush_interval,
        'rollover_size': rollover_size,
        'max_rollover_files': max_rollover_files,
        'compress': compress
    }

    sorted_new_obj = json_sort(new_obj)

    if not warnings:
        for obj in ret_obj['data']:
            del obj['file_size']
            sorted_obj = json_sort(obj)
            if sorted_obj == sorted_new_obj:
                id_exists = True

    return id_exists, warnings


def _check(isamAppliance, id, trace_id, trace_file_id):
    """
    Check to see if the file_id exists or not
    """
    ret_obj = get_all(isamAppliance, id)
    id_exists, warnings = False, ret_obj['warnings']

    if not warnings:
        for obj in ret_obj['data']:
            if obj['id'] == trace_id:
                filelist = get_list(isamAppliance, id, trace_id)
                for list_obj in filelist['data']:
                    if list_obj['id'] == trace_file_id:
                        logger.info("Found trace_id {0} trace_file_id '{1}'".format(trace_id, trace_file_id))
                        id_exists = True

    return id_exists, warnings



def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieve an overview of updates and licensing information
    """
    return isamAppliance.invoke_get("Retrieve an overview of updates and licensing information",
                                    "/updates/overview")


def get_licensing_info(isamAppliance, check_mode=False, force=False):
    """
    Retrieve the licensing information
    """
    return isamAppliance.invoke_get("Retrieve the licensing information",
                                    "/lum/is_licensed")



import json
import logging
import ibmsecurity.utilities.tools
from ibmsecurity.utilities import tools
from io import open

logger = logging.getLogger(__name__)

uri = "/extensions"
requires_modules = None
requires_version = "9.0.5.0"

try:
    basestring
except NameError:
    basestring = (str, bytes)


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Retrieve installed extensions list
    """
    return isamAppliance.invoke_get(
        "Retrieve installed extensions list",
        "{0}/".format(uri),
        requires_modules=requires_modules,
        requires_version=requires_version,
    )


def get(isamAppliance, extId, check_mode=False, force=False):
    """
    Retrieve installed extension by id
    """
    ret_obj = isamAppliance.create_return_object()
    extensions = get_all(isamAppliance)
    for obj in extensions["data"]:
        if obj["id"] == extId:
            ret_obj["data"] = obj
            break
    return ret_obj


def add(
    isamAppliance,
    extension,
    config_data=None,
    third_party_package=None,
    check_mode=False,
    force=False,
):
    """
    Installing an Extension

    :param isamAppliance:
    :param extension: path/filename to .ext file
    :param config_data: all the config data in a single string.  For example, "agentName:ISAM_Monitoring,ipAddress:10.10.10.10,port:9998"
    :                   or pass dictionary of parameters
    :param third_party_package: an array of the supporting files required.
    :param check_mode:
    :param force:
    :return:
    """

    if extension is None:
        warning_str = "extension is required for add"
        return isamAppliance.create_return_object(warnings=[warning_str])
    try:
        extId = inspect(isamAppliance, extension)
        logger.debug(f"We got {extId}")
    except Exception as e:
        warning_str = "Exception occurred: {0}".format(e)
        return isamAppliance.create_return_object(warnings=[warning_str])

    config_str = _get_config_data(extId, config_data)

    files = {}

    # files['extension_support_package'] = (tools.path_leaf(extension), open(extension, 'rb'))
    files["config_data"] = (None, config_str)

    if third_party_package:
        if isinstance(third_party_package, basestring):
            files["third_party_package"] = (
                tools.path_leaf(third_party_package),
                open(third_party_package, "rb"),
            )
        elif len(third_party_package) == 1:
            files["third_party_package"] = (
                tools.path_leaf(third_party_package[0]),
                open(third_party_package[0], "rb"),
            )
        else:
            counter = 0
            for file in third_party_package:
                third_party = "third_party_package{0}".format(counter)
                files[third_party] = (tools.path_leaf(file), open(file, "rb"))
                counter = counter + 1

    if check_mode:
        return isamAppliance.create_return_object(changed=True)

    return isamAppliance.invoke_post_files(
        "Installing an Extension",
        "{0}/activate".format(uri),
        [],
        files,
        requires_modules=requires_modules,
        requires_version=requires_version,
        json_response=False,
        data_as_files=True,
    )


def update(
    isamAppliance,
    extId,
    config_data=None,
    third_party_package=None,
    check_mode=False,
    force=False,
):
    """
    Update an existing installed extension

    :param isamAppliance:
    :param extId: extension id
    :param config_data: all the config data in a single string.  For example, "agentName:ISAM_Monitoring,ipAddress:10.10.10.10,port:9998"
    :param third_party_package: list of third_party files
    :param check_mode:
    :param force:
    :return:
    """

    if force or search(isamAppliance, extId=extId):
        if check_mode:
            return isamAppliance.create_return_object(changed=True)
        else:
            config_str = _get_config_data(extId, config_data)
            files = {}
            files["config_data"] = (None, config_str)

            if third_party_package:
                if isinstance(third_party_package, basestring):
                    files["third_party_package"] = (
                        tools.path_leaf(third_party_package),
                        open(third_party_package, "rb"),
                    )
                elif len(third_party_package) == 1:
                    files["third_party_package"] = (
                        tools.path_leaf(third_party_package[0]),
                        open(third_party_package[0], "rb"),
                    )
                else:
                    counter = 0
                    for file in third_party_package:
                        third_party = "third_party_package{0}".format(counter)
                        files[third_party] = (tools.path_leaf(file), open(file, "rb"))
                        counter = counter + 1

            return isamAppliance.invoke_post_files(
                "Update an Extension",
                "{0}/{1}".format(uri, extId),
                [],
                files,
                requires_modules=requires_modules,
                requires_version=requires_version,
                json_response=False,
                data_as_files=True,
            )

    return isamAppliance.create_return_object()


def set(
    isamAppliance,
    extension=None,
    extId=None,
    config_data=None,
    third_party_package=None,
    check_mode=False,
    force=False,
):

    # MUST have an extId to do an update.
    if extId and search(isamAppliance, extId):
        return update(
            isamAppliance=isamAppliance,
            extId=extId,
            config_data=config_data,
            third_party_package=third_party_package,
            check_mode=check_mode,
            force=True,
        )
    else:
        return add(
            isamAppliance=isamAppliance,
            extension=extension,
            config_data=config_data,
            third_party_package=third_party_package,
            check_mode=check_mode,
            force=force,
        )


def delete(isamAppliance, extId, check_mode=False, force=False):
    """
    Delete an installed extension
    """
    if force is True or search(isamAppliance, extId=extId):
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete(
                "Delete an installed extension", "{0}/{1}".format(uri, extId)
            )

    return isamAppliance.create_return_object(changed=False)


def inspect(isamAppliance, extension, check_mode=False, force=False):
    """
    Inspect the extension file to find the id for the extension.

    :param isamAppliance:
    :param extension:
    :param check_mode:
    :param force:
    :return:
    """
    obj = isamAppliance.invoke_post_files(
        "Inspect extension",
        "{0}/inspect".format(uri),
        [
            {
                "file_formfield": "extension_support_package",
                "filename": extension,
                "mimetype": "application/octet-stream",
            }
        ],
        {},
        json_response=False,
        data_as_files=False,
        ignore_error=True,
        requires_modules=requires_modules,
        requires_version=requires_version,
    )

    # Catch the errors here
    logger.debug("INSPECT\n" + str(obj))
    if obj.get("rc", 0) == 500:
        logger.debug(
            "Api does not allow to get the name from extension if it already exists"
        )
        return None
    else:
        m_obj = obj.get("data").decode("UTF-8")

        m_obj = m_obj.replace("<textarea>", "")
        m_obj = m_obj.replace("</textarea>", "")
        logger.debug("Returned data:\n" + m_obj)

        json_obj = json.loads(m_obj)

    return json_obj["id"]


def _get_config_data(extId, config_data):
    """
    Generate a JSON payload for activate/update
    """
    if config_data is None:
        return json.dumps({"extId": extId})
    if isinstance(config_data, basestring):
        return '{"extId": "' + extId + '",' + config_data + "}"
    else:
        config_data["extId"] = extId
        return json.dumps(config_data)


def search(isamAppliance, extId, check_mode=False, force=False):
    """
    Search for the extension
    """

    ret_obj = get_all(isamAppliance)
    if extId is None:
        return False

    for obj in ret_obj["data"]:
        if obj["id"] == extId:
            return True

    return False


def compare(isamAppliance1, isamAppliance2):
    """
    Compare extensions between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    if ret_obj1["data"]:
        del ret_obj1["data"][0]["date"]
    if ret_obj2["data"]:
        del ret_obj2["data"][0]["date"]

    return ibmsecurity.utilities.tools.json_compare(
        ret_obj1, ret_obj2, deleted_keys=["date"]
    )



import logging
import os.path
import zipfile
import shutil
from ibmsecurity.isam.aac.runtime_template import directory
from ibmsecurity.isam.aac.runtime_template import file
from ibmsecurity.utilities.tools import get_random_temp_dir, files_same_zip_content

logger = logging.getLogger(__name__)

uri = "/mga/template_files"

requires_modules = ["mga", "federation"]
requires_version = None


def export_file(isamAppliance, filename, check_mode=False, force=False):
    """
    Export all Runtime Template Files
    """
    import os.path

    if force is True or os.path.exists(filename) is False:
        if check_mode is False:  # No point downloading a file if in check_mode
            return isamAppliance.invoke_get_file(
                "Export all Runtime Template Files",
                "{0}/?export=true".format(uri),
                filename, no_headers=True, requires_modules=requires_modules,
                requires_version=requires_version)

    return isamAppliance.create_return_object()


def import_file(isamAppliance, filename, delete_missing=False, check_mode=False, force=False):
    """
    Import all Runtime Template Files with sync capability
    If delete_missing=True also sync provided file as master with the server.
    This will delete files on the server that exist there but are missing in the provided zip file
    To use import_file in additive mode, leave delete_missing=False
    """

    warnings = []

    if force is True or _check_import(isamAppliance, filename):
        if delete_missing is True:
            tempdir = get_random_temp_dir()
            tempfilename = "template_files.zip"
            tempfile =  os.path.join(tempdir, tempfilename)
            export_file(isamAppliance, tempfile)

            zServerFile = zipfile.ZipFile(tempfile)
            zClientFile = zipfile.ZipFile(filename)

            files_on_server = [];
            for info in zServerFile.infolist():
                files_on_server.append(info.filename)
            files_on_client = [];
            for info in zClientFile.infolist():
                files_on_client.append(info.filename)
            missing_client_files = [x for x in files_on_server if x not in files_on_client]
            
            if missing_client_files != []:
              logger.info("list all missing files in {}, which will be deleted on the server: {}.".format(filename, missing_client_files))

            for x in missing_client_files:                
                if x.endswith('/'):
                    search_dir= os.path.dirname(x[:-1]) + '/'
                    if search_dir not in missing_client_files:
                        logger.debug("delete directory on the server: {0}.".format(x))
                        delete(isamAppliance, x, "directory", check_mode=check_mode)
                else:
                    search_dir= os.path.dirname(x) + '/'
                    if search_dir not in missing_client_files:
                        logger.debug("delete file on the server: {0}.".format(x))
                        delete(isamAppliance, x, "file", check_mode=check_mode)
            zServerFile.close()
            shutil.rmtree(tempdir)

        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post_files(
                "Replace all Runtime Template Files",
                uri,
                [
                    {
                        'file_formfield': 'file',
                        'filename': filename,
                        'mimetype': 'application/octet-stream'
                    }
                ],
                {
                    "force": force
                }, json_response=False, requires_modules=requires_modules,
                requires_version=requires_version)

    return isamAppliance.create_return_object(warnings=warnings)

def _check_import(isamAppliance, filename):
    """
    Checks if runtime template zip from server and client differ
    :param isamAppliance:
    :param filename:
    :return:
    """

    tempdir = get_random_temp_dir()
    tempfilename = "template_files.zip"
    tempfile =  os.path.join(tempdir, tempfilename)
    export_file(isamAppliance, tempfile)
    
    if os.path.exists(tempfile):
      identical = files_same_zip_content(filename,tempfile)
    
      shutil.rmtree(tempdir)
      if identical:
          logger.info("runtime template files {} are identical with the server content. No update necessary.".format(filename))
          return False
      else:
          logger.info("runtime template files {} differ from the server content. Updating runtime template files necessary.".format(filename))
          return True
    else:
      logger.info("missing zip file from server. Comparison skipped.")
      return False

def check(isamAppliance, id, type, check_mode=False, force=False):
    ret_obj = None

    name = os.path.basename(id)
    path = os.path.dirname(id)

    if (type.lower() == 'directory'):
        ret_obj = directory._check(isamAppliance, id)
    elif (type.lower() == 'file'):
        ret_obj = file._check(isamAppliance, path, name)
    else:
        type = 'unknown'

    data = {
        'id': ret_obj,
        'path': path,
        'name': name,
        'type': type
    }

    return isamAppliance.create_return_object(data=data)


def delete(isamAppliance, id, type, check_mode=False, force=False):
    """
    Deleting a file or directory in the runtime template files directory

    :param isamAppliance:
    :param id:
    :param_type:
    :param check_mode:
    :param force:
    :return:
    """
    if (type.lower() == 'directory'):
        return directory.delete(isamAppliance, id)
    elif (type.lower() == 'file'):
        name = os.path.basename(id)
        path = os.path.dirname(id)
        return file.delete(isamAppliance, path, name)



import logging
from ibmsecurity.utilities.tools import json_sort

logger = logging.getLogger(__name__)

uri = "/mga/scim/configuration"
requires_modules = ["mga", "federation"]
requires_version = "9.0.2"

def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the current user profile SCIM configuration settings
    """
    return isamAppliance.invoke_get("Retrieving the current user profile SCIM configuration settings",
                                    "/mga/scim/configuration/urn:ietf:params:scim:schemas:core:2.0:User",
                                    requires_modules=requires_modules,
                                    requires_version=requires_version)


def set(isamAppliance, ldap_connection, search_suffix, user_suffix, ldap_object_classes, mappings, user_dn=None, connection_type=None, attrs_dir=None, enforce_password_policy=None, check_mode=False, force=False):
    """
    Updating the user profile SCIM configuration settings
    """
    ret_obj = get(isamAppliance)
    ret_obj = ret_obj['data']['urn:ietf:params:scim:schemas:core:2.0:User']

    new_obj = {'ldap_connection': ldap_connection,
               'user_suffix': user_suffix,
               'search_suffix': search_suffix,
               'ldap_object_classes': ldap_object_classes,
               'mappings': mappings,
               }


    if user_dn is not None:
        new_obj['user_dn'] = user_dn
    else:
        new_obj['user_dn'] = ret_obj['user_dn']

    if connection_type is not None:
        new_obj['connection_type'] = connection_type
    else:
        new_obj['connection_type'] = ret_obj['connection_type']

    if attrs_dir is not None:
        new_obj['attrs_dir'] = attrs_dir
    else:
        new_obj['attrs_dir'] = ret_obj['attrs_dir']

    if enforce_password_policy is not None:
        new_obj['enforce_password_policy'] = enforce_password_policy
    else:
        new_obj['enforce_password_policy'] = ret_obj['enforce_password_policy']

    obj1 = json_sort(ret_obj)
    obj2 = json_sort(new_obj)

    update_required = False
    if obj1 != obj2:
        update_required = True

    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put(
                "Updating the user profile SCIM configuration settings",
                "{0}/urn:ietf:params:scim:schemas:core:2.0:User".format(uri),
                new_obj, requires_modules=requires_modules,
                requires_version=requires_version)

    return isamAppliance.create_return_object()


import logging
import ibmsecurity.utilities.tools

try:
    basestring
except NameError:
    basestring = (str, bytes)

logger = logging.getLogger(__name__)

# URI for this module
uri = "/core/xforce_updates_cfg"
requires_modules = None
requires_version = None


def get(isamAppliance, check_mode=False, force=False):
    """
    Get Scheduled Security Updates
    """
    return isamAppliance.invoke_get("Get Scheduled Security Updates", uri, requires_modules=requires_modules,
                                    requires_version=requires_version)


def set(isamAppliance, enableAutoCheck=True, scheduleSettings=None, dailyFrequencySettings=None, schedule_type=None,
        check_mode=False, force=False):
    """
    Set Scheduled Security Update Settings
    Note: Sample values to be set:
        schedule_type: "day_or_week"
        scheduleSettings: {"day":"Sunday","time":"03:00"}
        dailyFrequencySettings: None
        OR
        schedule_type: "interval"
        dailyFrequencySettings: {"interval":60}
        scheduleSettings: None
        OR
        schedule_type: None
        dailyFrequencySettings: None
        scheduleSettings: None
    """
    update_required, json_data = _check(isamAppliance, enableAutoCheck=enableAutoCheck,
                                        scheduleSettings=scheduleSettings,
                                        dailyFrequencySettings=dailyFrequencySettings,
                                        schedule_type=schedule_type)

    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Set Scheduled Security Update Settings", uri, json_data,
                                            requires_modules=requires_modules,
                                            requires_version=requires_version)

    return isamAppliance.create_return_object()


def _check(isamAppliance, enableAutoCheck, scheduleSettings, dailyFrequencySettings, schedule_type):
    update_required = False
    json_data = {
        "config": {
            "enableAutoCheck": enableAutoCheck,
            "dailyFrequencySettings": None,
            "scheduleSettings": None
        }
    }
    ret_obj = get(isamAppliance)

    if schedule_type == 'day_or_week':
        json_data['config']['scheduleSettings'] = scheduleSettings
    elif schedule_type == 'interval':
        if 'interval' in dailyFrequencySettings and isinstance(dailyFrequencySettings['interval'], basestring):
            dailyFrequencySettings['interval'] = int(dailyFrequencySettings['interval'])
            logger.debug("fixing interval")
        json_data['config']['dailyFrequencySettings'] = dailyFrequencySettings

    sorted_json_data = ibmsecurity.utilities.tools.json_sort(json_data)
    logger.debug("Sorted input: {0}".format(sorted_json_data))
    sorted_ret_obj = ibmsecurity.utilities.tools.json_sort(ret_obj['data'])
    logger.debug("Sorted existing data: {0}".format(sorted_ret_obj))
    if sorted_ret_obj != sorted_json_data:
        logger.info("Changes detected, update needed.")
        json_data["schedule_type"] = schedule_type
        update_required = True

    return update_required, json_data


def compare(isamAppliance1, isamAppliance2):
    """
    Compare Scheduled Security Update Settings between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])
